%% CreateActin
%This is an algorithm to create a 3D actin net

%%% Version %%%
Version = 'D3_Stack1';
%%%%%%%%%%%%%%%

% Strang:
%  1  = strand length
%  2  = main direction
%  3  = max length
%  4+ = positions

% StrangAlt:
%  1  = strand length
%  2+ = positions

%% parameter
x = 80;
y = 80;
z = 80;
minDichte = 0.10;  %target density
maxDichte = 0.20;
chdp_min = 0.12;   %target chord length
chdp_max = 0.13;

%% variable initialisation
disp('initialisation');
tic
countAll = 0;
counter = 0;
counterAlt = 0;
Strang = cell(1,max([x y z])+1);
StrangAlt = Strang;
AktinNetz = zeros(x,y,z,2); %level 1: Actin/Fluid  | level 2: number of layers actin
Situation = 1;
GesamtDichte = 0;
TeilDichte = zeros(4,4,4);

%% Main program 
% === Start positions ===
for start=1:0.00005*x*y*z
  i = randi([1 x]);
  j = randi([1 y]);
  k = randi([1 z]);
  AktinNetz(i,j,k) = 1;
  counter = counter + 1;    
  Strang(counter,1) = {1};       %strand length
  Strang(counter,2) = {[randi([-1 1]),randi([-1 1]),randi([-1 1])]}; %strand main direction
  Strang(counter,3) = {ceil(randi([10 20])*max([x,y,z]))}; %max strand length
  Strang(counter,4) = {[i,j,k]}; %strand positions
end
clear start
disp('Start');

% === create new structures, until the density remains for a while in a certain level ===
while 1
  countAll = countAll + 1;
  % higher values --> sooner action is carried out
  if Situation == 1
    BuildRand = 0.000004;   %continue building strand     
    DestroyRand = 0;        %remove strand
    LuftLochZuRand = 0.002;	%fill gaps
    AktinRestWegRand = 0;   %open gaps
    GanzenStrangRand = 0;   %insert full length strands
    Verdickung = 8;
  end
  if Situation == 2
    BuildRand = 0;   
    DestroyRand = 0;
    LuftLochZuRand = 0.35;
    AktinRestWegRand = 0;
    GanzenStrangRand = 0;
    Verdickung = 2;
  end
  if Situation == 3
    BuildRand = 0.0001;   
    DestroyRand = 0;
    LuftLochZuRand = 0.02;
    AktinRestWegRand = 0;
    GanzenStrangRand = 0;
    Verdickung = 8;
  end
  if Situation == 4
    BuildRand = 0.0001;   
    DestroyRand = 0.0001; 
    LuftLochZuRand = 0.001;
    AktinRestWegRand = 0.001;
    GanzenStrangRand = 0.00001;
    Verdickung = 2;
  end
  if Situation == 5
    BuildRand = 0.0000001;
    DestroyRand = 0.001;
    LuftLochZuRand = 0.001;
    AktinRestWegRand = 0.01;
    GanzenStrangRand = 0;
    Verdickung = 2;
  end
  if Situation == 6
    BuildRand = 0;   
    DestroyRand = 0.0001;
    LuftLochZuRand = 0.01;
    AktinRestWegRand = 0.001;
    GanzenStrangRand = 0;
    Verdickung = 2;
  end
  if Situation == 7
    BuildRand = 0.00001;   
    DestroyRand = 0; 
    LuftLochZuRand = 0.0035; 
    AktinRestWegRand = 0;
    GanzenStrangRand = 0.0001   ;
    Verdickung = 6;
  end
  if Situation == 8
    BuildRand = 0.0000001;
    DestroyRand = 0.0001; 
    LuftLochZuRand = 0.004;
    AktinRestWegRand = 0.01;
    GanzenStrangRand = 0; 
    Verdickung = 2;  
  end

  % === Passing through all existing strains ===
  if Situation == 1 || Situation == 2 || Situation == 3 || Situation == 4 || Situation == 5 || Situation == 7 || Situation == 8
    s = 1;
    while s <= counter && ~isempty(Strang{1,1})
      %Change of direction
      if rand < 1/2   %Probability for change of direction
        %x-direction
        if rand < 1/3   %Probability for change of direction in x
          if (Strang{s,2}(1) == -1 || Strang{s,2}(1) == 1)
            Strang{s,2}(1) = 0;
          else
            if rand < 0.5
              Strang{s,2}(1) = -1;
            else
              Strang{s,2}(1) = 1;
            end
          end
        end
        %y-direction
        if rand < 1/3
          if Strang{s,2}(2) == -1 || Strang{s,2}(2) == 1
            Strang{s,2}(2) = 0;
          else
            if rand < 0.5
              Strang{s,2}(2) = -1;
            else
              Strang{s,2}(2) = 1;
            end
          end
        end
        %z-direction
        if rand < 1/3
          if Strang{s,2}(3) == -1 || Strang{s,2}(3) == 1
            Strang{s,2}(3) = 0;
          else
            if rand < 0.5
              Strang{s,2}(3) = -1;
            else
              Strang{s,2}(3) = 1;
            end
          end
        end
      end

      % == Checking whether the string is being continued or not ==
      l = Strang{s,1};  %current strand length
      %adjust position --> new pos. = old pos. + direction
      Strang(s,4+l) = {Strang{s,3+l}+Strang{s,2}}; 

      %Break off the strand at a certain strand length
      if l >= Strang{s,3}
        abbruch = 1;
      % check whether network outside borders
      elseif ~(Strang{s,4+l}(1) < 1 || Strang{s,4+l}(1) > x ...
              || Strang{s,4+l}(2) < 1 || Strang{s,4+l}(2) > y ...
              || Strang{s,4+l}(3) < 1 || Strang{s,4+l}(3) > z)
        AktinNetz(Strang{s,4+l}(1),Strang{s,4+l}(2),Strang{s,4+l}(3),1) = 1; %wrote "AktinNetz"
        AktinNetz(Strang{s,4+l}(1),Strang{s,4+l}(2),Strang{s,4+l}(3),2) = AktinNetz(Strang{s,4+l}(1),Strang{s,4+l}(2),Strang{s,4+l}(3),2) + 1; %Aktin Lagenanzahl erhoehen
        Strang(s,1) = {Strang{s,1}+1};  %increase length
        abbruch = 0; % if "abbruch=0" process was successful
      else
        abbruch = 1;
      end

      if abbruch == 1
        %Write strand to archive with reversed direction
        Strang{s,4+l} = []; %delete position
        counterAlt = counterAlt + 1;
        StrangAlt{counterAlt,1} = Strang{s,1};
        p = 2;
        while ~isempty(Strang{s,p+2})  %Transmit all positions of the strand
          % transfer position
          StrangAlt{counterAlt,p}(1) = Strang{s,p+2}(1);
          StrangAlt{counterAlt,p}(2) = Strang{s,p+2}(2);
          StrangAlt{counterAlt,p}(3) = Strang{s,p+2}(3);
          i = Strang{s,p+2}(1);
          j = Strang{s,p+2}(2);
          k = Strang{s,p+2}(3);

          verdicken = 0;
          for AnzahlVerdickung = 1:Verdickung 

            if AnzahlVerdickung == 1
              zusatzA = 1;
              zusatzB = 0;
            elseif AnzahlVerdickung == 2
              zusatzA = -1;
              zusatzB = 0;
            elseif AnzahlVerdickung == 3
              zusatzA = 0;
              zusatzB = 1;
            elseif AnzahlVerdickung == 4
              zusatzA = 0;
              zusatzB = -1;
            elseif AnzahlVerdickung == 5
              zusatzA = 1;
              zusatzB = 1;
            elseif AnzahlVerdickung == 6
              zusatzA = -1;
              zusatzB = -1;
            elseif AnzahlVerdickung == 7
              zusatzA = 1;
              zusatzB = -1;
            elseif AnzahlVerdickung == 8
              zusatzA = -1;
              zusatzB = 1;
            end
            zusatzCounter = AnzahlVerdickung;

            %z-level unchanged
            if (all(Strang{counter,2} == [1 0 0]) || all(Strang{counter,2} == [-1 0 0]) ...
                    || all(Strang{counter,2} == [1 1 0]) || all(Strang{counter,2} == [-1 1 0]) ...
                    || all(Strang{counter,2} == [1 -1 0]) || all(Strang{counter,2} == [-1 -1 0])) ...
                    && k ~= z && k ~= 1 && j ~= y && j ~= 1 %dont build strand outside the border
              AktinNetz(i,j+zusatzB,k+zusatzA,1) = 1;
              AktinNetz(i,j+zusatzB,k+zusatzA,2) = AktinNetz(i,j+zusatzB,k+zusatzA,2) + 1;
              StrangAlt(counterAlt+zusatzCounter,p) = {[i,j+zusatzB,k+zusatzA]}; %save position of 2. strand
              verdicken = 1;
            end
            %x-level unchanged
            if (all(Strang{counter,2} == [0 1 0]) || all(Strang{counter,2} == [0 -1 0]) ...
                    || all(Strang{counter,2} == [0 1 1]) || all(Strang{counter,2} == [0 -1 1]) ...
                    || all(Strang{counter,2} == [0 1 -1]) || all(Strang{counter,2} == [0 -1 -1])) ...
                    && i ~= x && i ~= 1 && k ~= z && k~= 1
              AktinNetz(i+zusatzA,j,k+zusatzB,1) = 1;
              AktinNetz(i+zusatzA,j,k+zusatzB,2) = AktinNetz(i+zusatzA,j,k+zusatzB,2) + 1;
              StrangAlt(counterAlt+zusatzCounter,p) = {[i+zusatzA,j,k+zusatzB]};
              verdicken = 1;
            end
            %y-level unchanged
            if (all(Strang{counter,2} == [0 0 1]) || all(Strang{counter,2} == [0 0 -1]) ...
                    || all(Strang{counter,2} == [1 0 1]) || all(Strang{counter,2} == [-1 0 1]) ...
                    || all(Strang{counter,2} == [1 0 -1]) || all(Strang{counter,2} == [-1 0 -1])) ...
                    && j ~= y && j ~= 1 && i ~= x && i ~= 1
              AktinNetz(i+zusatzB,j+zusatzA,k,1) = 1;
              AktinNetz(i+zusatzB,j+zusatzA,k,2) = AktinNetz(i+zusatzB,j+zusatzA,k,2) + 1;
              StrangAlt(counterAlt+zusatzCounter,p) = {[i+zusatzB,j+zusatzA,k]};
              verdicken = 1;
            end
            %no level unchanged
            if (all(Strang{counter,2} == [1 1 1]) || all(Strang{counter,2} == [-1 -1 -1]) ...
                    || all(Strang{counter,2} == [-1 1 1]) || all(Strang{counter,2} == [1 -1 -1]) ...
                    || all(Strang{counter,2} == [1 -1 1]) || all(Strang{counter,2} == [-1 1 -1]) ...
                    || all(Strang{counter,2} == [1 1 -1]) || all(Strang{counter,2} == [-1 -1 1])) ...
                    && i ~= x && j ~= y && k ~= z && i ~= 1 && j ~= 1 && k ~= 1
              AktinNetz(i+zusatzA,j,k+zusatzA,1) = 1;
              AktinNetz(i+zusatzA,j,k+zusatzA,2) = AktinNetz(i+zusatzA,j,k+zusatzA,2) + 1;
              StrangAlt(counterAlt+zusatzCounter,p) = {[i+zusatzA,j,k+zusatzA]};
              verdicken = 1;
            end
          end %for-loop "AnzahlVerdickung"
          p = p+1;
        end %While-loop "p"

        if verdicken == 1
            counterAlt = counterAlt + zusatzCounter;
        end

        %delete strand
        Strang(s,:) = [];
        s = s-1;
        counter = counter-1;
      end

      s = s+1;
    end %While-loop "s"
  end

  % === start new actin strands or break down old actin structures ===
  for Teiler_x = 0:3
    for Teiler_y = 0:3
      for Teiler_z = 0:3

        if TeilDichte(Teiler_x+1,Teiler_y+1,Teiler_z+1) <= GesamtDichte

          for i = 1+ceil(Teiler_x*x/4) : ceil(x/4+Teiler_x*x/4)
            for j = 1+ceil(Teiler_y*y/4) : ceil(y/4+Teiler_y*y/4)
              for k = 1+ceil(Teiler_z*z/4) : ceil(z/4+Teiler_z*z/4)

                % new strands
                if (AktinNetz(i,j,k,1) == 1 && rand < BuildRand) || (AktinNetz(i,j,k,1) == 0 && rand < 100*1/100*BuildRand)
                  AktinNetz(i,j,k,1) = 1;
                  AktinNetz(i,j,k,2) = AktinNetz(i,j,k,2) + 1;
                  counter = counter + 1;    
                  Strang(counter,1) = {1};       %strand length
                  Strang(counter,2) = {[randi([-1 1]),randi([-1 1]),randi([-1 1])]}; %strand main direction
                  Strang(counter,3) = {ceil(randi([10 20])*max([x,y,z]))};                                    
                  Strang(counter,4) = {[i,j,k]}; %strand position
                end

                % delete old strands
                if rand < DestroyRand && counterAlt > 0
                  sAlt = randi([1 counterAlt]); %pick a strand at random

                  if isempty(StrangAlt{sAlt,1})
                    StrangAlt(sAlt,:) = [];
                    counterAlt = counterAlt-1;
                    continue
                  end

                  l = StrangAlt{sAlt,1};
                  Position1 = {StrangAlt{sAlt,1+l}(1),StrangAlt{sAlt,1+l}(2),StrangAlt{sAlt,1+l}(3)};  %Determine strand position
                  AktinNetz(Position1{:},2) = AktinNetz(Position1{:},2) - 1;   %Lower strand layers by one
                  if AktinNetz(Position1{:},2) == 0
                    AktinNetz(Position1{:},1) = 0; %wrote "AktinNetz"
                  end
                  StrangAlt{sAlt,1+l} = [];
                  StrangAlt{sAlt,1} = StrangAlt{sAlt,1}-1;

                  %If strand is completely dismantled --> delete
                  if isempty(StrangAlt{sAlt,2})
                    StrangAlt(sAlt,:) = [];
                    counterAlt = counterAlt-1;
                  end
                end

              end %k
            end %j
          end %i

        end %if "TeilerDichte"

      end %Teiler_z
    end %Teiler_y
  end %Teiler_x

  % === fill or open gaps ===
  for chordRichtung = 4:7
    if chordRichtung == 1
      a = 1;
      b = 0;
      c = 0;
    elseif chordRichtung == 2
      a = 0;
      b = 1;
      c = 0;
    elseif chordRichtung == 3
      a = 0;
      b = 0;
      c = 1;
    elseif chordRichtung == 4
      a = 1;
      b = 1;
      c = 0;
    elseif chordRichtung == 5
      a = 1;
      b = 0;
      c = 1;
    elseif chordRichtung == 6
      a = 0;
      b = 1;
      c = 1;
    elseif chordRichtung == 7
      a = 1;
      b = 1;
      c = 1;
    end

    for Teiler_x = 0:3
      for Teiler_y = 0:3
        for Teiler_z = 0:3

          %Density too low
          if TeilDichte(Teiler_x+1,Teiler_y+1,Teiler_z+1) <= GesamtDichte && (LuftLochZuRand ~= 0 || GanzenStrangRand ~= 0)

            for i = 1+ceil(Teiler_x*x/4) : ceil(x/4+Teiler_x*x/4)
              for j = 1+ceil(Teiler_y*y/4) : ceil(y/4+Teiler_y*y/4)
                for k = 1+ceil(Teiler_z*z/4) : ceil(z/4+Teiler_z*z/4)

                  if i>2 && i<x-1 && j>2 && j<y-1 && k>2 && k<z-1

                    %gap size 1
                    if AktinNetz(i,j,k,1) == 0 &&  AktinNetz(i+a,j+b,k+c,1) == 1 &&  AktinNetz(i-a,j-b,k-c,1) == 1 && rand < LuftLochZuRand
                      AktinNetz(i,j,k,1) = 1;
                    end
                    %gap size 2
                    if AktinNetz(i,j,k,1) == 0 &&  AktinNetz(i+a,j+b,k+c,1) == 0 &&  AktinNetz(i-a,j-b,k-c,1) == 1 ...
                            && AktinNetz(i+a+a,j+b+b,k+c+c,1) == 1 && rand < LuftLochZuRand
                      AktinNetz(i,j,k,1) = 1;
                      AktinNetz(i+a,j+b,k+c,1) = 1;
                    elseif AktinNetz(i,j,k,1) == 0 &&  AktinNetz(i+a,j+b,k+c,1) == 1 &&  AktinNetz(i-a,j-b,k-c,1) == 0 ...
                            && AktinNetz(i-a-a,j-b-b,k-c-c,1) == 1 && rand < LuftLochZuRand
                      AktinNetz(i,j,k,1) = 1;
                      AktinNetz(i-a,j-b,k-c,1) = 1;
                    end

                    try %outside border
                      if AktinNetz(i,j,k,1) == 0 && rand < GanzenStrangRand
                        s = 1;
                        while AktinNetz(i+s*a,j+s*b,k+s*c,1) == 0 && s < ceil(2/10000 * x*y*z) 
                          s = s+1;
                        end
                        if s >= 8
                          counterAlt = counterAlt + 1;
                          StrangAlt(counterAlt,1) = {s-4};   %strand length
                          for t=3:s-2
                            AktinNetz(i+t*a,j+t*b,k+t*c,1) = 1;
                            AktinNetz(i+t*a,j+t*b,k+t*c,2) = AktinNetz(i+t*a,j+t*b,k+t*c,2) + 1;
                            StrangAlt(counterAlt,1+t-2) = {[i+t*a,j+t*b,k+t*c]}; %strand position
                          end
                        end
                      end
                    catch
                      if s >= 8
                        counterAlt = counterAlt + 1;
                        StrangAlt(counterAlt,1) = {s-4};   %strand length
                        for t=3:s-2
                          AktinNetz(i+t*a,j+t*b,k+t*c,1) = 1;
                          AktinNetz(i+t*a,j+t*b,k+t*c,2) = AktinNetz(i+t*a,j+t*b,k+t*c,2) + 1;
                          StrangAlt(counterAlt,1+t-2) = {[i+t*a,j+t*b,k+t*c]}; %strand position
                        end
                      end
                    end %try

                  end %if

                end %k
              end %j
            end %i

          end %if TeilDichte

          %densitiy too high
          if TeilDichte(Teiler_x+1,Teiler_y+1,Teiler_z+1) >= GesamtDichte && AktinRestWegRand ~= 0
            for i = 1+ceil(Teiler_x*x/4) : ceil(x/4+Teiler_x*x/4)
              for j = 1+ceil(Teiler_y*y/4) : ceil(y/4+Teiler_y*y/4)
                for k = 1+ceil(Teiler_z*z/4) : ceil(z/4+Teiler_z*z/4)

                  if i>2 && i<x-1 && j>2 && j<y-1 && k>2 && k<z-1

                    %gap size 1
                    if AktinNetz(i,j,k,1) == 1 &&  AktinNetz(i+a,j+b,k+c,1) == 0 &&  AktinNetz(i-a,j-b,k-c,1) == 0 && rand < AktinRestWegRand
                      AktinNetz(i,j,k,1) = 0;
                    end
                    %gap size 2
                    if AktinNetz(i,j,k,1) == 1 &&  AktinNetz(i+a,j+b,k+c,1) == 1 &&  AktinNetz(i-a,j-b,k-c,1) == 0 ...
                            && AktinNetz(i+a+a,j+b+b,k+c+c,1) == 0 && rand < AktinRestWegRand
                      AktinNetz(i,j,k,1) = 0;
                      AktinNetz(i+a,j+b,k+c,1) = 0;
                    elseif AktinNetz(i,j,k,1) == 1 &&  AktinNetz(i+a,j+b,k+c,1) == 0 &&  AktinNetz(i-a,j-b,k-c,1) == 1 ...
                            && AktinNetz(i-a-a,j-b-b,k-c-c,1) == 0 && rand < AktinRestWegRand
                      AktinNetz(i,j,k,1) = 0;
                      AktinNetz(i-a,j-b,k-c,1) = 0;
                    end

                  end %if 

                end %k
              end %j
            end %i

          end %if TeilDichte

        end %Teiler_z
      end %Teiler_y
    end %Teiler_x

  end


  % === Reduce and save matrix if necessary ===
  if Situation == 9
    AktinNetz = ~AktinNetz;
    toc
    if x>15 && y>15 && z>15
      ZwischenSpeicher = AktinNetz(6:x-5,6:y-5,6:z-5,1);
      clear AktinNetz
      AktinNetz = ZwischenSpeicher;
      uisave('AktinNetz',['V' Version '_' num2str(x-10) '_' num2str(y-10) '_' num2str(z-10)]);
    else
      uisave('AktinNetz',['V' Version '_' num2str(x) '_' num2str(y) '_' num2str(z)]);
    end

    answer = questdlg('Save created actin network as VTK file for visualisation with ParaView?', ...
        'Visualisierung','Ja','Nein','Nein');
    if strcmp(answer,'Ja')
      path = 'D:\';
      vtkwrite([path 'V' Version '_' num2str(x-10) '_' num2str(y-10) '_' num2str(z-10) '.vtk'], 'structured_points', 'AktinNetz', AktinNetz)
    end

    break

  end
% Save created actin network as vtk for visualisation with ParaView?

  % === Check density and chord lengths and adjust abort counter ===
  % only check the inner area of the cell, as the density on the outside is very low
  % --> outside will be cut off later
  if x>15 && y>15 && z>15
    sumAktin = sum(sum(sum(AktinNetz(6:x-5,6:y-5,6:z-5,1))));
    Dichte = sumAktin/((x-10)*(y-10)*(z-10));
    sumAktin = sum(sum(sum(AktinNetz(:,:,:,1))));
    GesamtDichte = sumAktin/(x*y*z);
    [~,~,~,chd_x,chd_y,chd_z] = chord_length_3d(AktinNetz(6:x-5,6:y-5,6:z-5,1));
    chdp_x = chd_x/(x-10);
    chdp_y = chd_y/(y-10);
    chdp_z = chd_z/(z-10);
  else
    sumAktin = sum(sum(sum(AktinNetz(:,:,:,1))));
    Dichte = sumAktin/(x*y*z);
    [~,~,~,chd_x,chd_y,chd_z] = chord_length_3d(AktinNetz(:,:,:,1));
    chdp_x = chd_x/x;
    chdp_y = chd_y/y;
    chdp_z = chd_z/z;
  end

  for Teiler_x = 0:3
    for Teiler_y = 0:3
      for Teiler_z = 0:3
        sumAktin = sum(sum(sum(AktinNetz( ...
            1+ceil(Teiler_x*x/4) : ceil(x/4+Teiler_x*x/4), ...
            1+ceil(Teiler_y*y/4) : ceil(y/4+Teiler_y*y/4), ...
            1+ceil(Teiler_y*y/4) : ceil(y/4+Teiler_y*y/4)))));
        TeilDichte(Teiler_x+1,Teiler_y+1,Teiler_z+1) = sumAktin/(x*y*z/64); 
      end
    end
  end


  % === Control values and set situation ===
  if     Dichte < minDichte && chdp_x > chdp_max
    Situation = 1;
  elseif Dichte < minDichte && chdp_x < chdp_min
    Situation = 2;
  elseif Dichte < minDichte && chdp_x > chdp_min && chdp_x < chdp_max
    Situation = 3;
  elseif Dichte > maxDichte && chdp_x > chdp_max
    Situation = 4;
  elseif Dichte > maxDichte && chdp_x < chdp_min
    Situation = 5;
  elseif Dichte > maxDichte && chdp_x > chdp_min && chdp_x < chdp_max
    Situation = 6;
  elseif Dichte > minDichte && Dichte < maxDichte && chdp_x > chdp_max
    Situation = 7;
  elseif Dichte > minDichte && Dichte < maxDichte && chdp_x < chdp_min
    Situation = 8;
  elseif Dichte > minDichte && Dichte < maxDichte && chdp_x > chdp_min && chdp_x < chdp_max
    Situation = 9;
  end


% === Display of the current values ==
disp(['countAll = ' num2str(countAll) '   counter = ' num2str(counter)...
    '   counterAlt = ' num2str(counterAlt) ...
    '   Dichte = ' num2str(Dichte) '   chdp_x = ' num2str(chdp_x)...
    '   Situation = ' num2str(Situation)]);
   
end

clear x y z i j k l minDichte maxDichte Dichte TeilDichte GesamtDichte sumAktin
clear Strang StrangLaengeMax counter s p StrangAlt counterAlt sAlt
clear DestroyRand BuildRand Version AktinRestWegRand LuftLochZuRand GanzenStrangRand
clear chdp_min chdp_max chd_x chd_y chd_z chdp_x chdp_y chdp_z
clear Situation a b c chordRichtung countAll ZwischenSpeicher
clear Position1 Position2 t Teiler_x Teiler_y Teiler_z
clear AnzahlVerdickung zusatzCounter verdicken zusatzA zusatzB abbruch Verdickung
clear answer path 